# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.
"""Class for building a Swagger spec from a notebook-defined API."""

import os

class SwaggerSpecBuilder(object):
    """Builds a Swagger specification.

    Parameters
    ----------
    kernel_spec : str
        Name of the kernel language

    Attributes
    ----------
    cell_parser : services.cell.parser.APICellParser
        Finds cell annotations to add to the spec
    value : dict
        Python object representation of the Swagger spec
    """
    def __init__(self, cell_parser):
        self.cell_parser = cell_parser
        self.value = self.cell_parser.get_default_api_spec()

    def add_cell(self, cell_source):
        """Parses a notebook cell for API annotations.

        If found, adds the HTTP verb and endpoint to the Swagger spec.

        Parameters
        ----------
        cell_source : str
            Source code from a notebook
        """
        if self.cell_parser.is_api_cell(cell_source):
            path_name, verb = self.cell_parser.get_cell_endpoint_and_verb(cell_source)
            path_value = self.cell_parser.get_path_content(cell_source)
            if not path_name in self.value['paths']:
                self.value['paths'][path_name] = {}
            self.value['paths'][path_name][verb.lower()] = path_value

    def set_default_title(self, path):
        """Stores the root of a notebook filename as the API title, if one is
        not already present.

        Parameters
        ----------
        path : url
            Path to the notebook file defining the API
        """
        if 'info' in self.value and 'title' not in self.value['info']:
            basename = os.path.basename(path)
            self.value['info']['title'] = basename.split('.')[0] if basename.find('.') > 0 else basename

    def build(self):
        """Gets the specification.

        Returns
        -------
        dict
            Python object representation of the Swagger spec
        """
        return self.value



# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.
"""Tornado handler that returns the swagger specification of a
notebook-http mode, notebook-defined API.
"""

import tornado.web
import json
from .builders import SwaggerSpecBuilder
from ...mixins import TokenAuthorizationMixin, CORSMixin, JSONErrorsMixin

class SwaggerSpecHandler(TokenAuthorizationMixin,
                         CORSMixin,
                         JSONErrorsMixin,
                         tornado.web.RequestHandler):
    """Handles requests for the Swagger specification of a notebook defined
    API.
    """
    output = None

    def initialize(self, notebook_path, source_cells, cell_parser):
        """Builds the spec for the notebook-defined API.

        Parameters
        ----------
        notebook_path : str
            Path to the notebook, used to set the API title
        source_cells : list
            code cells
        cell_parser : obj
            Parser that will return endpoints and documentation
        """
        if self.output is None:
            spec_builder = SwaggerSpecBuilder(cell_parser)
            for source_cell in source_cells:
                if 'source' in source_cell:
                    spec_builder.add_cell(source_cell['source'])
            spec_builder.set_default_title(notebook_path)
            SwaggerSpecHandler.output = json.dumps(spec_builder.build())

    def get(self, **kwargs):
        """Responds with the spec in JSON format."""
        self.set_header('Content-Type', 'application/json')
        self.set_status(200)
        self.finish(self.output)



print("import: 'kernel_gateway'")
import kernel_gateway

print("import: 'kernel_gateway.base'")
import kernel_gateway.base

print("import: 'kernel_gateway.jupyter_websocket'")
import kernel_gateway.jupyter_websocket

print("import: 'kernel_gateway.notebook_http'")
import kernel_gateway.notebook_http

print("import: 'kernel_gateway.notebook_http.cell'")
import kernel_gateway.notebook_http.cell

print("import: 'kernel_gateway.notebook_http.swagger'")
import kernel_gateway.notebook_http.swagger

print("import: 'kernel_gateway.services'")
import kernel_gateway.services

print("import: 'kernel_gateway.services.kernels'")
import kernel_gateway.services.kernels

print("import: 'kernel_gateway.services.kernelspecs'")
import kernel_gateway.services.kernelspecs

print("import: 'kernel_gateway.services.sessions'")
import kernel_gateway.services.sessions

